@@ -8,16 +8,17 @@ use apexl\Io\includes\Routes;
 use apexl\Io\includes\System;
 use apexl\Io\modules\display\components\BasicLink;
 use apexl\Io\modules\display\components\ButtonBar;
+use apexl\Io\modules\display\components\CardEntityFilteredDisplayTable;
 use apexl\Io\modules\display\components\ColWrapper;
 use apexl\Io\modules\display\components\ContentTitle;
 use apexl\Io\modules\display\components\EntityFilteredDisplayTable;
+use apexl\Io\modules\display\components\form\CheckboxField;
+use apexl\Io\modules\display\components\form\Form;
+use apexl\Io\modules\display\components\form\InputField;
 use apexl\Io\modules\display\components\FormComponent;
 use apexl\Io\modules\display\components\genericComponents;
 use apexl\Io\modules\display\components\RowWrapper;
 use apexl\Io\modules\display\services\Render;
-use apexl\Io\modules\formbuilder\entities\formEntity;
-use apexl\Io\modules\formbuilder\includes\checkboxField;
-use apexl\Io\modules\formbuilder\includes\inputField;
 use apexl\Io\modules\user\entities\roleEntity;
 use apexl\Io\modules\user\entities\userEntity;
 use apexl\Io\modules\user\enums\permissions\Role;
@@ -70,23 +71,17 @@ class userController extends Controller
      */
     public function userSettingsPage(Request $request, Response $response)
     {
-        $form = (new FormComponent())
-            ->src(Routes::getRoutePattern('user.display.forms.userSettings'))
-            ->setID('register');
+        $form = $this->userEditForm($request, $response, ['id' => $this->currentUser->id]);
+
+        //$form = (new FormComponent())
+        //    ->src(Routes::getRoutePattern('user.display.forms.userSettings'))
+        //    ->setID('register');
 
         $this->render::setPageTitle('User Settings');
 
         return System::asJson($response, $this->render::build([$form]));
     }
 
-    /**
-     * @return Response
-     * @throws \Exception
-     */
-    public function userSettingsForm(Request $request, Response $response)
-    {
-        return $this->userEditForm($request, $response, ['id' => $this->currentUser->id]);
-    }
 
     /**
      * @param  array  $args
@@ -95,32 +90,30 @@ class userController extends Controller
      */
     public function userEditForm(Request $request, Response $response, array $args = [])
     {
-        $form = $this->primaryAccountForm();
-        $form->setSubmitText('Submit');
+
 
         //set form field values
         $user = new userEntity();
         if (isset($args[$user->primaryKey])) {
             $user->load($args[$user->primaryKey]);
+        }
+
+        $form = $this->primaryAccountForm(false, $user);
+        $form->setSubmitText('Submit');
+        if (isset($args[$user->primaryKey])) {
             $form->setMethod('PUT');
         }
 
         $form->setActionUrl(Routes::getRoutePattern('userEntity.put'));
 
         if ($this->currentUser->isAllowed(User::CHANGE_ACTIVE_STATE)) {
-            $activeField = (new checkboxField('active'))->setLabel('Active')->setValue($user->active);
+            $activeField = (new CheckboxField('active'))->setLabel('Active')->setValue($user->active ?? 0);
         } else {
-            $activeField = (new inputField('active'))->setInputType('hidden');
+            $activeField = (new InputField('active'))->setInputType('hidden')->setValue($user->active ?? 0);
         }
 
         if ($id = $args[$user->primaryKey] ?? false) {
             $form->setActionUrl(Routes::getRoutePattern($user->getEntityName().'.put', ['id' => $id]));
-
-            $form->addField($form->getFormField('first_name')->setValue($user->first_name));
-            $form->addField($form->getFormField('last_name')->setValue($user->last_name));
-            $form->addField($form->getFormField('email')->setValue($user->email));
-
-            $activeField->setChecked($user->active);
         }
 
         $form->addField($activeField);
@@ -131,13 +124,12 @@ class userController extends Controller
             //for users that can't change roles, show labels for those that they do have.
             $showLabel = false;
             if ($this->currentUser->isAllowed(Role::UPDATE)) {
-                $roleField = (new checkboxField('role_'.$role->id))
+                $roleField = (new CheckboxField('role_'.$role->id))
                     ->setLabel($role->name)
-                    ->setChecked($user->hasRole($role))
                     ->setValue($user->hasRole($role));
             } else {
                 $showLabel = true;
-                $roleField = (new inputField('role_'.$role->id));
+                $roleField = (new InputField('role_'.$role->id));
                 $roleField->setInputType('hidden');
             }
             if (($user->id ?? false) && $user->hasRole($role)) {
@@ -153,34 +145,30 @@ class userController extends Controller
         //allow other modules to modify form.
         $form = Hook::processHook('displayFormRender', $form, $user, 'userEditForm');
 
-        //set role values
-        $this->output::addResponse($request, $form->getBuiltFormArray());
-
-        return System::asJson($response);
+        return $form;
     }
 
     /**
      * @param  bool  $passwordRequired
      */
-    protected function primaryAccountForm($passwordRequired = false): formEntity
+    protected function primaryAccountForm($passwordRequired = false, $user=null): Form
     {
-        $form = new formEntity();
-        $form->setId('login');
+        $form = new Form();
         $form->setMethod('post');
         $form->setActionUrl(Routes::getRoutePattern('userEntity.post'));
 
         $form->addField(
-            (new inputField('first_name'))->setLabel('First Name')->setPlaceholder('First Name')->isRequired()
+            (new InputField('first_name'))->setLabel('First Name')->setPlaceholder('First Name')->isRequired()->setValue(isset($user->first_name) ? $user->first_name : '')
         );
-        $form->addField((new inputField('last_name'))->setLabel('Surname')->setPlaceholder('Surname')->isRequired());
+        $form->addField((new InputField('last_name'))->setLabel('Surname')->setPlaceholder('Surname')->isRequired()->setValue(isset($user->last_name) ? $user->last_name : ''));
         $form->addField(
-            (new inputField('email'))->setLabel('Email')->setPlaceholder('Email')->setInputType('email')->isRequired()
+            (new InputField('email'))->setLabel('Email')->setPlaceholder('Email')->setInputType('email')->isRequired()->setValue(isset($user->email) ? $user->email : '')
         );
 
-        $password = (new inputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType(
+        $password = (new InputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType(
             'password'
         )->addValidation("minLength", ["min" => 6]);
-        $confirmPass = (new inputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder(
+        $confirmPass = (new InputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder(
             'Confirm Password'
         )->setInputType('password')->addValidation("minLength", ["min" => 6]);
         if ($passwordRequired) {
@@ -208,6 +196,11 @@ class userController extends Controller
         $this->render::setActive("noUI");
         $this->render::setMetaTitle("Login | ".($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
         $components = [];
+
+        if ($newComponents = Hook::processHook('login_page_components_before_form', $components)){
+            $components = $newComponents;
+        }
+
         if (isset($args['loginMessageType']) && isset($this->config->app->loginMessageTypes->{$args['loginMessageType']})) {
             $components[] = (new ContentTitle())->addContent(
                 $this->config->app->loginMessageTypes->{$args['loginMessageType']}
@@ -215,11 +208,43 @@ class userController extends Controller
         }
         $redirect = $request->getQueryParams()['redirect'] ?? null;
         $redirect = $redirect ? sprintf('?redirect=%s', htmlentities($redirect)) : null;
-        $components[] = (new FormComponent())->src(
-            Routes::getRoutePattern('user.display.forms.login').$redirect
-        )
+
+        $form = (new Form())
             ->title(System::getVariable('user_login_description_text') ?? "Sign in with your email address.")
-            ->addClass('login-form')->setID('login');
+            ->setMethod('post')
+            ->setActionUrl(Routes::getRoutePattern('user.login'))
+            ->setSubmitText(System::getVariable('user_login_submit_text') ?? 'Sign in')
+            ->addClass('login-form');
+        if (System::getVariable('user_login_include_clear_button') ?? true) {
+            $form->includeClearButton()->setClearButtonText(
+                System::getVariable('user_login_include_clear_button_text') ?? 'Cancel'
+            );
+        }
+
+        if ($request) {
+            $redirect = ($request->getQueryParams()['redirect'] ?? null);
+            if ($redirect) {
+                $form->addField(
+                    (new InputField('redirect'))
+                        ->setInputType('hidden')
+                        ->setValue($redirect)
+                );
+            }
+        }
+
+        $form->addField(
+            (new InputField('username'))->setPlaceholder(
+                System::getVariable('user_login_email_placeholder') ?? 'Enter your email address'
+            )->setLabel('Email')->setInputType('email')->isRequired()
+        );
+        $form->addField(
+            (new InputField('password'))->setPlaceholder(
+                System::getVariable('user_login_password_placeholder') ?? 'Enter your password'
+            )->setLabel('Password')->setInputType('password')->isRequired()
+        );
+
+        $components[] = $form;
+
         $components[] = (new BasicLink())->addClass('forgottenPasswordLink')->addRoute(
             Routes::getRoutePattern('user.display.forgot-password')
         )->addText(System::getVariable('user_login_forgot_password_text') ?? 'Forgot password?');
@@ -227,6 +252,10 @@ class userController extends Controller
             Routes::getRoutePattern('user.display.register')
         )->addText(System::getVariable('user_login_register_text') ?? 'No account? Register.');
 
+        if ($newComponents = Hook::processHook('login_page_components_after_form', $components)){
+            $components = $newComponents;
+        }
+
         return System::asJson($response, $this->render::build($components));
     }
 
@@ -277,7 +306,11 @@ class userController extends Controller
 
         $rowWrapper = (new RowWrapper())->addClass('d-block');
         $rowWrapper->addComponent(
-            (new FormComponent())->src(Routes::getRoutePattern('user.display.forms.forgottenPassword'))
+            (new Form())
+                ->setMethod('post')
+                ->setActionUrl(Routes::getRoutePattern('user.forgot-password'))
+                ->setSubmitText('Get Reset Link')
+                ->addField((new inputField('email'))->setPlaceholder(System::getVariable('user_login_email_placeholder') ?? 'Enter your email address')->setLabel('Email Address')->setInputType('email')->isRequired())
                 ->title("Enter your email to reset your password.")
                 ->addClass('forgotten-password-form')->setID('forgottenPassword')
         );
@@ -362,12 +395,31 @@ class userController extends Controller
      */
     public function userList(Request $request, Response $response)
     {
-        $filterForm = (new EntityFilteredDisplayTable((new userEntity()), [], false))->addEditButton('userEntity.display.put');
-        $filterForm->addFilterForm('user.display.forms.filters.userFilter');
+        //$filterForm = (new EntityFilteredDisplayTable((new userEntity()), [], false))->addEditButton('userEntity.display.put');
+        //$filterForm->addFilterForm('user.display.forms.filters.userFilter');
+
+        $components = [];
+
+        $currentUser = currentUser::getCurrentUser();
+        if ($currentUser && ($currentUser->isAllowed(User::CREATE))) {
+            $createButton = (new BasicLink())
+                ->addText('Create User')
+                ->addRoute(Routes::getRoutePattern('userEntity.display.post'))
+                ->addClass('btn btn-primary');
+            $components[] = $createButton;
+        }
+
+        $components[] = (new CardEntityFilteredDisplayTable(new userEntity(), [], false, 'dropmenu'))
+            ->displayPagination(FALSE)
+            ->displayShortTotals(FALSE)
+            ->hideIdColumn()
+            ->addDropMenuItem('userEntity.display.put', 'Edit')
+            ->addTitle('User List')
+        ;
 
         //render
         $this->render::setPageTitle('User List');
-        $this->output::addResponse($request, $this->render::build($filterForm));
+        $this->output::addResponse($request, $this->render::build($components));
 
         return System::asJson($response);
     }
@@ -381,9 +433,7 @@ class userController extends Controller
             $title = 'Update '.$user->getNiceName();
         }
 
-        $form = (new FormComponent())->setID('userCreate')->src(
-            Routes::getRoutePattern('user.display.forms.createUpdate', $args)
-        );
+        $form = $this->userEditForm($request, $response, $args);
         $this->render::setPageTitle($title);
         $this->output::addResponse($request, $this->render::build([genericComponents::dashboardBlockWrapper([$form])]));
 
@@ -404,9 +454,7 @@ class userController extends Controller
             $title = 'Edit ('.$id.') '.$userEntity->getNiceName();
         }
 
-        $form = (new FormComponent())->setID('roleCreate')->src(
-            Routes::getRoutePattern('user.display.forms.createUpdate', $args)
-        );
+        $form = $this->userEditForm($request, $response, $args);
         $this->render::setPageTitle($title);
         $this->output::addResponse($request, $this->render::build($form));
 
@@ -426,10 +474,19 @@ class userController extends Controller
             $match = $user->getForgottenPasswordLink($hash);
             if (!empty($match) && ((time() - $match['created']) <= 86400 && $match['used'] == 0)) {
                 $valid = true;
-                $components[] = (new FormComponent())
-                    ->src(Routes::getRoutePattern('user.display.forms.resetPassword', ['hash' => $hash]))
+                $form = new Form();
+                $form-->setSubmitText('Reset Password Now')
+                    ->setMethod('post')
+                    ->setActionUrl(Routes::getRoutePattern('user.resetPassword'))
+                    ->addField((new InputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType('password')->addValidation("minLength", ["min" => 6])->isRequired())
+                    ->addField((new InputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder('Confirm Password')->setInputType('password')->addValidation("minLength", ["min" => 6])->isRequired())
+                    ->addField((new InputField('hash'))->setInputType('hidden')->setValue($args['hash'])->isRequired())
                     ->title("Reset your password using the fields below:")
-                    ->addClass('forgotten-password-form')->setID('forgottenPassword');
+                    ->addClass('forgotten-password-form');
+
+
+                $components[] = $form;
+
                 $components[] = (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(Routes::getRoutePattern('user.display.login'))->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login');
             }
         }
@@ -462,7 +519,6 @@ class userController extends Controller
     public function resetPasswordForm(Request $request, Response $response, $args)
     {
         $form = new formEntity();
-        $form->setId('reset-password-form');
         $form->setSubmitText('Reset Password Now');
         $form->setMethod('post');
         $form->setActionUrl(Routes::getRoutePattern('user.resetPassword'));
